-----------------------------------------------------------------------------------------------
-- Client Lua Script for QuestCleanup
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- QuestCleanup Module Definition
-----------------------------------------------------------------------------------------------
local QuestCleanup = {} 
 
local qcOptions = {
	"abandonTrivial",
	"abandonEasy",
	"abandonSimple",
	"skipMuseum",
	"skipTradeskills",
	"skipImbuement"
}

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function QuestCleanup:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function QuestCleanup:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "QuestCleanup"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- QuestCleanup OnLoad
-----------------------------------------------------------------------------------------------
function QuestCleanup:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("QuestCleanup.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- QuestCleanup OnDocLoaded
-----------------------------------------------------------------------------------------------
function QuestCleanup:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "QuestCleanupForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("questcleanup", "OnQuestCleanupOn", self)
		Apollo.RegisterSlashCommand("qcconfig", "OnQuestCleanupOnConfig", self)
		Apollo.RegisterSlashCommand("qc", "OnQuestCleanupOn", self)


		-- Do additional Addon initialization here
	end
end

function QuestCleanup:RefreshConfig()
	if self.abandonSimple ~= nil then
		self.wndMain:FindChild("CleanupColor3"):SetCheck(self.abandonSimple)
	end
	
	if self.abandonEasy ~= nil then
		self.wndMain:FindChild("CleanupColor2"):SetCheck(self.abandonEasy)
	end
	if self.abandonTrivial ~= nil then
		self.wndMain:FindChild("CleanupColor1"):SetCheck(self.abandonTrivial)
	end
	
	if self.skipMuseum ~= nil then
		self.wndMain:FindChild("chkMuseum"):SetCheck(self.skipMuseum)
	end
	if self.skipTradeskills ~= nil then
		self.wndMain:FindChild("chkTradeskills"):SetCheck(self.skipTradeskills)
	end
	if self.skipImbuement ~= nil then
		self.wndMain:FindChild("chkImbuement"):SetCheck(self.skipImbuement)
	end
end

-----------------------------------------------------------------------------------------------
-- QuestCleanup Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/qcconfig"
function QuestCleanup:OnQuestCleanupOnConfig()
	self.wndMain:Invoke() -- show the window
	self:RefreshConfig()
end

function QuestCleanup:OnConfigure()
	self.wndMain:Invoke()
	self:RefreshConfig()
end

function QuestCleanup:OnSave(eType)
	local toSave = {}
	for id,property in ipairs(qcOptions) do
			toSave[property] = self[property]		
	end
	
	return toSave
end

function QuestCleanup:OnRestore(eType, t)
	for id,property in ipairs(qcOptions) do
		if t[property] ~= nil then
			self[property] = t[property]
		end
	end
	
end

--Returns 'true' if the quest can be abandoned according to the addon's configuration
function QuestCleanup:SafeToAbandon(queQuest)
	local maxLevelToAbandon = 1 --Always abandon at least "Trivial" quests
	
	if self.abandonSimple then
		maxLevelToAbandon = 3
	elseif self.abandonEasy then
		maxLevelToAbandon = 2
	end
	
	local queColor = queQuest:GetColoredDifficulty()

	--First check if it's over the configured level range
	if queColor > maxLevelToAbandon then
		return false
	end
	
	--Then check if the game will even let us abandon this quest, some can't be abandoned
	if not queQuest:CanAbandon() then
		return false
	end
	
	--Now check the category name of the quest, support for "Skip Tradeskill Quests"
	local queCategory = queQuest:GetCategory():GetTitle()
	
	if queCategory == 'Tradeskills' and self.skipTradeskills then
		return false
	end
	
	--[[TODO:  This should just be a list of quest names for museum quests, not string parsing,
				but I don't know all the museum quest names
	if self.skipMuseum then			
		local museumQuests = {['SomeQuest'] = true,
								['SomeOtherQuest'] = true,
								['etc'] = true}
		local questName = queQuest:GetTitle()
		
		if museumQuests[questName] ~= nil then
			return false
		end
	end
	]]--
	--Now check the description for the word "Museum," support for "Skip Museum Quests"
	local queShortDesc = queQuest:GetObjectiveShortDescription(0)
	if self.skipMuseum and string.find(string.lower(queShortDesc), "museum") then
		return false
	end
	
	--Now check the quest title for "IMBUEMENT," support for "Skip Imbuement Quests"
	local queTitle = queQuest:GetTitle()
	if self.skipImbuement and string.find(string.lower(queTitle), "imbuement") then
		return false
	end
	
	return true
end

-- on SlashCommand "/questcleanup"
function QuestCleanup:OnQuestCleanupOn()
	local numQuests = QuestLib.GetCount()
	local numAbandoned = 0
	for i=0, numQuests, 1 do
		queQuest = QuestLib.GetQuest(i)
		if queQuest == nil then
			break
		end
		
		if self:SafeToAbandon(queQuest) then
				queQuest:Abandon()
				numAbandoned = numAbandoned + 1		
		end
	end
	ChatSystemLib.PostOnChannel(2, 'Quests abandoned: ' .. numAbandoned)
end

-----------------------------------------------------------------------------------------------
-- QuestCleanupForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function QuestCleanup:OnOK()
	self.abandonTrivial = self.wndMain:FindChild("CleanupColor1"):IsChecked()
	self.abandonEasy = self.wndMain:FindChild("CleanupColor2"):IsChecked()
	self.abandonSimple = self.wndMain:FindChild("CleanupColor3"):IsChecked()
	self.skipMuseum = self.wndMain:FindChild("chkMuseum"):IsChecked()
	self.skipTradeskills = self.wndMain:FindChild("chkTradeskills"):IsChecked()
	self.skipImbuement = self.wndMain:FindChild("chkImbuement"):IsChecked()
	
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function QuestCleanup:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- QuestCleanup Instance
-----------------------------------------------------------------------------------------------
local QuestCleanupInst = QuestCleanup:new()
QuestCleanupInst:Init()
